package main

import (
	"flag"
	"fmt"
	"html/template"
	"io/ioutil"
	"net/http"

	"git.thirdmartini.com/thirdmartini/vessel"
)

func main() {
	extractFlag := flag.String("extract-bundle", "", "--extract-bundle=<extract point>")
	localContentFlag := flag.String("local-content", "", "serve content from local site/")
	flag.Parse()

	if *extractFlag != "" {
		b, err := vessel.Self()
		if err != nil {
			panic(err)
		}

		err = b.Decompress(*extractFlag)
		if err != nil {
			panic(err)
		}
		return
	}

	b, err := vessel.Self()
	if err != nil {
		panic(err)
	}

	// We're using an overlay FS so that we can transparently prefer things
	// locally over our built bundle in the source for testing.
	// this lets us make quick changes to the site/ content without having to regenerate a bundle
	o := vessel.NewOverlayFS()
	if *localContentFlag != "" {
		o.Append(http.FileSystem(http.Dir(*localContentFlag)))
	}
	o.Append(b.FileSystem())

	http.Handle("/", http.FileServer(o))
	http.HandleFunc("/list", func(w http.ResponseWriter, r *http.Request) {
		f, err := o.Open("/templates/list.html")
		if err != nil {
			w.WriteHeader(http.StatusNotFound)
			w.Write([]byte(err.Error()))
			return
		}
		defer f.Close()

		data, err := ioutil.ReadAll(f)
		if err != nil {
			w.WriteHeader(http.StatusNotFound)
			w.Write([]byte(err.Error()))
			return
		}

		t, err := template.New("list").Parse(string(data))
		if err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			w.Write([]byte(err.Error()))
			return
		}
		w.WriteHeader(http.StatusOK)
		files := b.List()
		t.Execute(w, files)
	})

	http.HandleFunc("/listraw", func(w http.ResponseWriter, r *http.Request) {
		w.WriteHeader(http.StatusOK)
		files := b.List()

		for _, n := range files {
			fmt.Fprintf(w, "%s\n", n)
		}
	})

	http.ListenAndServe(":8080", nil)
}
