package vessel

import (
	golog "log"
)

var log Logger = &NilLogger{}

// Logger provides an interface to get at log data for vessel
type Logger interface {
	Log(format string, a ...interface{})
}

// NilLogger dumps all logs to nil
type NilLogger struct {
}

func (l NilLogger) Log(format string, a ...interface{}) {}

// DebugLogger defines s simple logger that dumps files to stdout
type DebugLogger struct {
}

// Log implements Logger.Log interface
func (l DebugLogger) Log(format string, a ...interface{}) {
	golog.Printf(format, a...)
}

// SetLogger sets the logger to use for vessel
func SetLogger(l Logger) {
	if l != nil {
		log = l
	}
}
