package vessel

import (
	"fmt"
	"os"
	"path/filepath"
	"strings"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
)

func TestVessel(t *testing.T) {
	ename := "./test.bundle"

	SetLogger(DebugLogger{})

	f, err := os.Create(ename)
	assert.Nil(t, err)
	f.Close()
	defer os.Remove(ename)

	b, err := Build(ename, "example", true, func(path string) bool {
		base := filepath.Base(path)
		if base == ".DS_Store" {
			return false
		}
		if strings.HasSuffix(base, ".zip") {
			return false
		}
		return true
	})
	assert.Nil(t, err)
	assert.NotNil(t, b)

	fs := b.FileSystem()
	assert.NotNil(t, fs)

	files, err := fs.Dir("/")
	assert.Nil(t, err)
	assert.Equal(t, 3, len(files))
	fmt.Println("E----", len(files))

	fmt.Println("S----")
	files, err = fs.Dir("/site")
	assert.Nil(t, err)
	assert.Equal(t, 4, len(files))
	fmt.Println("E----", len(files))

	stat, err := fs.Stat("/site")
	assert.Nil(t, err)
	assert.Equal(t, true, stat.IsDir())

	stat, err = fs.Stat("/site/static")
	assert.Nil(t, err)
	assert.Equal(t, true, stat.IsDir())

	stat, err = fs.Stat("/site/templates/list.html")
	assert.Nil(t, err)
	assert.Equal(t, false, stat.IsDir())

	lstat, err := os.Stat("example/" + "/site/templates/list.html")
	assert.NoError(t, err)
	assert.Equal(t, lstat.ModTime().UTC(), stat.ModTime().UTC())

	now := time.Now()
	b.WithTimeOverride(now)
	fs = b.FileSystem()
	assert.NotNil(t, fs)

	stat, err = fs.Stat("/site/templates/list.html")
	assert.Nil(t, err)

	assert.NoError(t, err)
	assert.Equal(t, now.UTC(), stat.ModTime().UTC())
}
